package com.takensoft.ai_lms.common.confing;

import com.takensoft.ai_lms.common.filter.JwtFilter;
import com.takensoft.ai_lms.common.filter.LoginFilter;
import com.takensoft.ai_lms.common.util.JwtUtil;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.security.authentication.AuthenticationManager;
import org.springframework.security.config.annotation.authentication.configuration.AuthenticationConfiguration;
import org.springframework.security.config.annotation.web.builders.HttpSecurity;
import org.springframework.security.config.annotation.web.configuration.EnableWebSecurity;
import org.springframework.security.config.http.SessionCreationPolicy;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.security.web.SecurityFilterChain;
import org.springframework.security.web.authentication.UsernamePasswordAuthenticationFilter;
import org.springframework.web.cors.CorsConfiguration;
import org.springframework.web.cors.CorsConfigurationSource;
import org.springframework.web.cors.UrlBasedCorsConfigurationSource;

import java.util.Collections;
import java.util.List;

@Configuration
@EnableWebSecurity
public class SecurityConfig {

    private final AuthenticationConfiguration authenticationConfiguration;
    private final JwtUtil jwtUtil;
    private final CommonConfig commonConfig;

    private static String FRONT_URL; // 프론트 경로
    private static long JWT_ACCESSTIME; // access 토큰 유지 시간
    private static int COOKIE_TIME; // 쿠키 유지 시간

    public SecurityConfig(AuthenticationConfiguration authenticationConfiguration, JwtUtil jwtUtil, CommonConfig commonConfig,
                          @Value("${frontUrl}") String fUrl, @Value("${spring.jwt.accessTime}") long aTime, @Value("${spring.jwt.refreshTime}") long rTime, @Value("${cookie.time}") int ctime
    ) {
        this.authenticationConfiguration = authenticationConfiguration;
        this.jwtUtil = jwtUtil;
        this.commonConfig = commonConfig;

        this.FRONT_URL = fUrl;
        this.JWT_ACCESSTIME = aTime;
        this.COOKIE_TIME = ctime;
    }

    // AuthenticationManager Bean 등록
    @Bean
    public AuthenticationManager authenticationManager(AuthenticationConfiguration configuration) throws Exception {
        return configuration.getAuthenticationManager();
    }

    @Bean
    public SecurityFilterChain securityFilterChain(HttpSecurity http) throws Exception {
        http.csrf((auth) -> auth.disable()); // CSRF 비활성화
        http.formLogin((auth) -> auth.disable()); // 폼 로그인 비활성화
        http.httpBasic((auth) -> auth.disable()); // HTTP Basic 비활성화
        http.sessionManagement((auth) -> auth.sessionCreationPolicy(SessionCreationPolicy.STATELESS)); // 세션 비활성화
        http.cors((auth) -> auth.configurationSource(corsConfigurationSource())); // CORS 설정
        http.authorizeHttpRequests((auth) -> auth
                .requestMatchers( "/auth/**", "/auth/login.json").permitAll() // /user/** 경로는 모두 허용
                .requestMatchers("/swagger-ui/**", "/v3/api-docs/**").permitAll() // swagger 진입 허용
                .requestMatchers("/test/**").permitAll()
                .requestMatchers("/text/**").permitAll() // 지문 정보 진입 허용
                .requestMatchers("/schedule/**").permitAll() // 학습일정 정보 진입 허용
                .requestMatchers("/studentInfo/**").permitAll() // 학생 정보 진입 허용(민수)
                .requestMatchers("/board/**").permitAll() // 게시판 정보 진입 허용
                .requestMatchers("/book/**").permitAll() // 교재 정보 진입 허용
                .requestMatchers("/file/**").permitAll() // 파일 정보 진입 허용
                .requestMatchers("/classes/**").permitAll() // 반 정보 진입 허용
                .requestMatchers("/classBook/**").permitAll() // 반 - 책 정보 진입 허용
                .requestMatchers("/unit/**").permitAll() // 단원 정보 진입 허용
                .requestMatchers("/photo/**").permitAll() // 사진 정보 진입 허용
                .requestMatchers("/photoMng/**").permitAll() // 사진첩 정보 진입 허용
                .requestMatchers("/wordbook/**").permitAll()
                .requestMatchers("/problem/**").permitAll() // 문제 정보 진입 허용
                .requestMatchers("/problemLog/**").permitAll() // 문제 풀이 로그 정보 진입 허용
                .requestMatchers("/evaluation/**").permitAll() // 단원 평가 정보 진입 허용
                .requestMatchers("/evalProblem/**").permitAll() // 단원 평가 문제 진입 허용
                .requestMatchers("/unitLearning/**").permitAll() // 로드맵 정보 진입 허용
                .requestMatchers("/unit/**").permitAll()
                .requestMatchers("/photo/**").permitAll()
                .requestMatchers("/wordbook/**").permitAll() // 단어장 정보 진입 허용
                .requestMatchers("/word/**").permitAll() // 단어 정보 진입 허용
                .requestMatchers("/survey/**").permitAll() // 설문 조사 진입 허용
                .requestMatchers("/api/**").permitAll() // Google Cloud api 진입 허용
                .requestMatchers("/userclass/**").permitAll() // 반별 학생 맵핑 진입 허용
                .requestMatchers("/userunit/**").permitAll() // 사용자 학습 진행 정보 진입 허용
                .anyRequest().authenticated()); // 나머지 경로는 인증 필요

        // jwt 필터 처리 적용
        http.addFilterBefore(new JwtFilter(jwtUtil, commonConfig), LoginFilter.class); // 토큰 검증 필터
        http.addFilterAt(new LoginFilter(authenticationManager(authenticationConfiguration), jwtUtil, commonConfig, JWT_ACCESSTIME), UsernamePasswordAuthenticationFilter.class);

        return http.build();
    }

    // 패스워드 암호화
    @Bean
    public BCryptPasswordEncoder bCryptPasswordEncoder() {
        return new BCryptPasswordEncoder();
    }

    // CORS 설정
    @Bean
    public CorsConfigurationSource corsConfigurationSource() {
        CorsConfiguration configuration = new CorsConfiguration();
        configuration.setAllowedOrigins(Collections.singletonList(FRONT_URL));
        configuration.setAllowedMethods(List.of("GET", "POST", "PUT", "DELETE", "PATCH", "OPTIONS"));
        configuration.setAllowedHeaders(List.of("*"));
        configuration.setExposedHeaders(Collections.singletonList("Authorization"));

        UrlBasedCorsConfigurationSource source = new UrlBasedCorsConfigurationSource();
        source.registerCorsConfiguration("/**", configuration);

        return source;
    }

    // inmemory방식 유저 설정
//    @Bean
//    public InMemoryUserDetailsManager userDetailsService() {
//        UserDetails user = User.withDefaultPasswordEncoder()
//                .username("user")
//                .password("password")
//                .roles("USER")
//                .build();
//        return new InMemoryUserDetailsManager(user);
//    }

}