package com.takensoft.ai_lms.common.filter;

import com.takensoft.ai_lms.common.confing.CommonConfig;
import com.takensoft.ai_lms.common.util.ErrorResponse;
import com.takensoft.ai_lms.common.util.JwtUtil;
import com.takensoft.ai_lms.lms.auth.vo.UserAuthorVO;
import com.takensoft.ai_lms.lms.auth.vo.UserVO;
import io.jsonwebtoken.ExpiredJwtException;
import io.jsonwebtoken.JwtException;
import jakarta.servlet.FilterChain;
import jakarta.servlet.http.HttpServletRequest;
import jakarta.servlet.http.HttpServletResponse;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.security.authentication.InsufficientAuthenticationException;
import org.springframework.security.authentication.UsernamePasswordAuthenticationToken;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Component;
import org.springframework.web.filter.OncePerRequestFilter;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.List;

@Component
@RequiredArgsConstructor
public class JwtFilter extends OncePerRequestFilter  {

    private final JwtUtil jwtUtil;
    private final CommonConfig commonConfig;

    @Override
    protected void doFilterInternal(HttpServletRequest request, HttpServletResponse response, FilterChain filterChain) throws jakarta.servlet.ServletException, IOException {
        // 헤더에서 access에 대한 토큰을 꺼냄
        String accessToken = request.getHeader("Authorization");
        // 토큰이 없다면 다음 필터로 넘김
        if (accessToken == null) {
            filterChain.doFilter(request, response);
            return;
        }
        try {
            // 토큰 만료 여부 검증
            if (jwtUtil.isExpired(accessToken)) {
                throw new JwtException("Token expired");
            }
            // 토큰에서 페이로드 확인[ 발급시 명시 ]
            String category = jwtUtil.getCategory(accessToken);
            if (!category.equals("Authorization")) {
                throw new JwtException("Wrong Token");
            }
            // 토큰에서 사용자 정보 추출
            UserVO user = new UserVO();
            List<UserAuthorVO> author = jwtUtil.getRoles(accessToken);
            user.setLoginId(jwtUtil.getLoginId(accessToken));
            user.setUsid(jwtUtil.getUsid(accessToken));
            user.setAuthor(author);

            // 시큐리티 컨텍스트에 사용자 정보 설정
            Authentication authToken = new UsernamePasswordAuthenticationToken(user, null, user.getAuthorities());
            SecurityContextHolder.getContext().setAuthentication(authToken);
            // 다음 필터로 이동
            filterChain.doFilter(request, response);
        } catch (JwtException | InsufficientAuthenticationException e) {
            // 토큰 검증 실패 시 클라이언트에게 에러 응답을 위한 객체 생성
            ErrorResponse errorResponse = new ErrorResponse();
            // 에러 응답 메시지 설정
            if (e instanceof ExpiredJwtException) {
                errorResponse.setMessage("Token expired");
            } else {
                errorResponse.setMessage(e.getMessage());
            }
            errorResponse.setPath(request.getRequestURI()); // 오류 경로 설정
            errorResponse.setError(HttpStatus.UNAUTHORIZED.getReasonPhrase()); // 오류 원인 설정
            errorResponse.setStatus(HttpStatus.UNAUTHORIZED.value()); // 상태코드 설정
            errorResponse.setTimestamp(LocalDateTime.now()); // 응답 시간 설정

            // 응답 헤더 설정 및 json 응답 전송
            response.setContentType(MediaType.APPLICATION_JSON_VALUE);
            response.setStatus(HttpStatus.UNAUTHORIZED.value());
            response.getOutputStream().write(commonConfig.getObjectMapper().writeValueAsBytes(errorResponse));
        }
    }
}
