package com.takensoft.ai_lms.lms.classes.web;

import com.google.gson.Gson;
import com.google.gson.JsonObject;
import com.takensoft.ai_lms.lms.classes.service.ClassService;
import com.takensoft.ai_lms.lms.classes.vo.ClassVO;
import io.swagger.v3.oas.annotations.Operation;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;

@RestController
@RequiredArgsConstructor
@Slf4j
@RequestMapping(value="/classes")
public class ClassController {

    private final ClassService classService;

    /**
     * @author  정다정
     * @since   2024.07.25
     * param   ClassDAO
     * @return
     * @throws  Exception
     *
     * 반 조회
     */
    @PostMapping("/selectClass.json")
    @Operation(summary = "반 조회")
    public String selectClass(@RequestBody Map<String, Object> params) throws Exception {
        Gson gson = new Gson();
        JsonObject response = new JsonObject();

        try {
            String userId = params.get("userId").toString();
            List<ClassVO> result = classService.selectClass(userId);

            response.addProperty("status", "success");
            response.add("data", gson.toJsonTree(result));

            return gson.toJson(response);
        } catch (Exception e) {
            e.printStackTrace();

            response.addProperty("status", "error");
            response.addProperty("message", e.getMessage());

            return gson.toJson(response);
        }
    }

    /**
     * @author  정다정
     * @since   2024.07.25
     * param   ClassDAO
     * @return
     * @throws  Exception
     *
     * 반 생성
     */
    @PostMapping("/insertClass.json")
    @Operation(summary = "반 생성")
    public String insertClass(@RequestBody ClassVO classVO) throws Exception{
        Gson gson = new Gson();
        JsonObject response = new JsonObject();

        try{
            // userId 또는 sclsNm(반 이름)이 비어있으면 fail
            if (classVO.getUserId() == null || classVO.getUserId().isEmpty() ||
                    classVO.getSclsNm() == null || classVO.getSclsNm().isEmpty()) {
                response.addProperty("status", "fail");
                response.addProperty("message", "userId 또는 반 이름이 비어있습니다.");
                return gson.toJson(response);
            }

            int result = classService.insertClass(classVO);

            if(result == -1){
                response.addProperty("status", "fail");
                return gson.toJson(response);
            }

            response.addProperty("status", "success");
            return gson.toJson(response);
        } catch (Exception e) {
            e.printStackTrace();

            response.addProperty("status", "error");
            response.addProperty("message", e.getMessage());

            return gson.toJson(response);
        }
    }

    /**
     * @author  정다정
     * @since   2024.07.25
     * param   ClassDAO
     * @return
     * @throws  Exception
     *
     * 반 수정
     */
    @PostMapping("/updateClass.json")
    @Operation(summary = "반 수정")
    public String updateClass(@RequestBody ClassVO classVO) throws Exception{
        Gson gson = new Gson();
        JsonObject response = new JsonObject();

        try{
            // 반 아이디(scls_id)가 존재하는지 확인
            if (!classService.existsClassById(classVO.getSclsId())) {
                // 반 아이디(scls_id)가 db에 존재 하지 않으면 수정 실패
                response.addProperty("status", "fail");
                response.addProperty("message", "존재 하지 않는 반 아이디(sclsId)입니다.");
                return gson.toJson(response);
            }

            int result = classService.updateClass(classVO);

            if(result == -1){
                response.addProperty("status", "fail");
                return gson.toJson(response);
            }

            response.addProperty("status", "success");
            return gson.toJson(response);
        } catch (Exception e) {
            e.printStackTrace();

            response.addProperty("status", "error");
            response.addProperty("message", e.getMessage());

            return gson.toJson(response);
        }
    }


    /**
     * @author  정다정
     * @since   2024.07.25
     * param   ClassDAO
     * @return
     * @throws  Exception
     *
     * 반 삭제
     */
    @PostMapping("/deleteClass.json")
    @Operation(summary = "반 삭제")
    public String deleteClass(@RequestBody Map<String, Object> params) throws Exception{
        Gson gson = new Gson();
        JsonObject response = new JsonObject();

        try {
            String sclsId = params.get("sclsId").toString();
            classService.deleteClass(sclsId);
            response.addProperty("status", "success");
            return gson.toJson(response);
        } catch (Exception e) {
            e.printStackTrace();

            response.addProperty("status", "error");
            response.addProperty("message", e.getMessage());

            return gson.toJson(response);
        }
    }


}
